state = 1
font = love.graphics.newFont(14)

supports = {
  fb = false,
  nonPo2Fb = false,
  nonPo2Img = false
}

maxSize = 2
maxNonPo2 = 3

love.graphics.setFont(font)

-- tests
function love.load()  
  -- Po2 Framebuffers
  status, fb = pcall(love.graphics.newFramebuffer, 2, 2)
  if status and type(fb) == 'userdata' then supports.fb = true end

  -- non-Po2 Framebuffers
  if supports.fb then
    status, fb = pcall(love.graphics.newFramebuffer, 3, 3)
    if status and type(fb) == 'userdata' then supports.nonPo2Fb = true end
  end

  imgData = love.image.newImageData(150, 150)
  imgData:mapPixel(function() return 255, 0, 0, 255 end)
  img = love.graphics.newImage(imgData)
end

function love.draw()
  local str = "You support:\n"
  
  if not supports.fb and not supports.nonPo2Fb and not supports.nonPo2Img then
    str = str .. "  Nothing"
  else
    if supports.fb then str = str .. "  Framebuffers\n" end
    if supports.nonPo2Fb then str = str .. "  Non-Po2 Framebuffers\n" end
    if supports.nonPo2Img then str = str .. "  Non-Po2 Images\n" end
    
    if supports.fb and maxSize ~= 2 then 
      str = str .. ("  Framebuffers up to the size %s x %s\n"):format(maxSize, maxSize)
    end
    
    if supports.nonPo2Fb and maxNonPo2 ~= 3 then
      str = str .. ("  Power of 3 Framebuffers up to the size %s x %s\n"):format(maxNonPo2, maxNonPo2)
    end
  end
  
  love.graphics.draw(img, 325, 225)
  love.graphics.print(str, 50, 20)
  
  if state == 1 then
    love.graphics.print("Please press 'y' if you see a red square. Otherwise press 'n'.", 50, 500)
  elseif state == 2 then
    love.graphics.print("Now press space to test the maximum size of framebuffers", 50, 500)
  elseif state == 3 then
    love.graphics.print("Hold on... Creating framebuffers.", 50, 500)
    state = 4
  elseif state == 4 then
    if not supports.fb then return end
    
    for i = 1, 14 do
      status, fb = pcall(love.graphics.newFramebuffer, maxSize, maxSize)
      if not status or type(fb) ~= 'userdata' then break end
      maxSize = maxSize * 2
    end
    
    if supports.nonPo2Fb then
      for i = 1, 14 do
        status, fb = pcall(love.graphics.newFramebuffer, maxNonPo2, maxNonPo2)
        if not status or type(fb) ~= 'userdata' then break end
        maxNonPo2 = maxNonPo2 * 3
      end
    end
    
    state = 5
  elseif state == 5 then
    love.graphics.print("Done! Now go and post your results at:\nhttp://love2d.org/forums/viewtopic.php?f=3&t=2907\nThanks!", 50, 500)
  end
end

function love.keypressed(key, unicode)
  if state == 1 and key == 'y' then
    supports.nonPo2Img = true
    state = 2
  end
  
  if state == 1 and key == 'n' then state = 2 end
  if state == 2 and key == ' ' then state = 3 end
end
